# Standard library
import unittest

# Local
from canonicalwebteam import image_template
from urllib.parse import quote


asset_url = (
    "https://assets.ubuntu.com/" "v1/479958ed-vivid-hero-takeover-kylin.jpg"
)
encoded_asset_url = quote(asset_url)
non_asset_url = (
    "https://dashboard.snapcraft.io/site_media/appmedia/"
    "2018/10/Screenshot_from_2018-10-26_14-20-14.png"
)
non_hostname_url = "/static/images/Screenshot_from_2018-10-26_14-20-14.png"


class TestImageTemplate(unittest.TestCase):
    def test_returns_string(self):
        markup = image_template(
            url=asset_url,
            alt="test",
            width="1920",
            height="1080",
            hi_def=False,
        )
        self.assertTrue(isinstance(markup, str))

    def test_attributes(self):
        markup = image_template(
            url=asset_url,
            alt="test",
            width="1920",
            height="1080",
            hi_def=False,
            attrs={"id": "test", "title": "test title"},
        )
        self.assertIn('id="test"', markup)
        self.assertIn('title="test title"', markup)

    def test_classes(self):
        markup = image_template(
            url=asset_url,
            alt="test",
            width="1920",
            height="1080",
            hi_def=False,
            attrs={"class": "test-title"},
        )
        # Check custom class exists
        self.assertIn('class="test-title"', markup)

    def test_optional_lazy(self):
        markup = image_template(
            url=asset_url,
            alt="test",
            width="1920",
            height="1080",
            loading="auto",
            attrs={"class": "test-title"},
            hi_def=False,
        )
        # Check lazyload class is not present
        self.assertIn('class="test-title"', markup)

    def test_optional_fill(self):
        markup = image_template(
            url=asset_url,
            alt="test",
            width="1920",
            height="1080",
            loading="auto",
            fill=True,
            hi_def=False,
        )
        # Check c_fill is present
        self.assertIn("c_fill", markup)

    def test_e_sharpen(self):
        markup = image_template(
            url=asset_url,
            alt="test",
            width="1920",
            height="1080",
            loading="auto",
            fill=True,
            e_sharpen=True,
            hi_def=False,
        )
        # Check e_sharpen is present
        self.assertIn("e_sharpen", markup)

    def test_hi_def(self):
        markup = image_template(
            url=non_asset_url,
            alt="test",
            width="1920",
            height="1080",
            hi_def=True,
        )
        markup_asset = image_template(
            url=asset_url, alt="test", width="1920", height="1080", hi_def=True
        )

        # Check the markup includes srcset
        self.assertTrue(markup.find("srcset="))
        self.assertTrue(markup.find("data-srcset"))
        # Check x2 is present
        self.assertTrue(markup.find("x2"))
        # Check width and height are double
        self.assertTrue(markup.find("3840"))
        self.assertTrue(markup.find("2160"))

        self.assertTrue(markup_asset.find("srcset="))
        self.assertTrue(markup_asset.find("data-srcset"))
        self.assertTrue(markup_asset.find("x2"))
        self.assertTrue(markup_asset.find("w%3D3840%26h%3D2160"))

    def test_height_is_optional(self):
        image = image_template(
            url=non_asset_url,
            alt="test",
            width="1920",
            hi_def=True,
        )

        self.assertNotIn("height=", image)
        self.assertNotIn("h_auto", image)


if __name__ == "__main__":
    unittest.main()



#! /usr/bin/env python3

# Core
from setuptools import setup

setup(
    name="canonicalwebteam.image-template",
    version="1.5.0",
    author="Canonical webteam",
    author_email="webteam@canonical.com",
    url=(
        "https://github.com/canonical-web-and-design/"
        "canonicalwebteam.image-template"
    ),
    packages=["canonicalwebteam", "canonicalwebteam.image_template"],
    include_package_data=True,
    description=("Generate <img> markup block for an image."),
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    install_requires=["jinja2>=2"],
    test_suite="tests",
)



# Standard library
import os
import sys
from urllib.parse import urlparse

# Packages
from jinja2 import Environment, FileSystemLoader

parent_dir = os.path.dirname(os.path.dirname(os.path.realpath(__file__)))
env = Environment(loader=FileSystemLoader(parent_dir + "/templates"))
template = env.get_template("image_template.html")


def image_template(
    url,
    alt,
    hi_def,
    width,
    height=None,
    fill=False,
    e_sharpen=False,
    loading="lazy",
    fmt="auto",
    attrs={},
):
    """
    Generate image markup
    """

    url_parts = urlparse(url)

    # Default cloudinary optimisations
    # https://cloudinary.com/documentation/image_transformations

    cloudinary_options = [
        "f_" + str(fmt),
        "q_auto",  # Auto optimise quality
        "fl_sanitize",  # Sanitize SVG content
    ]

    if e_sharpen:
        cloudinary_options.append("e_sharpen")

    # If the original image does not match the requested
    # ratio set crop and fill see
    # https://cloudinary.com/documentation/image_transformation_reference#crop_parameter
    if fill:
        cloudinary_options.append("c_fill")

    if not url_parts.netloc:
        raise Exception("url must contain a hostname")

    std_def_cloudinary_options = cloudinary_options.copy()
    hi_def_cloudinary_options = cloudinary_options.copy()

    std_def_cloudinary_options.append("w_" + str(width))

    if height is not None:
        std_def_cloudinary_options.append("h_" + str(height))

    if hi_def:
        hi_def_cloudinary_options.append("w_" + str(int(width) * 2))

        if height is not None:
            hi_def_cloudinary_options.append("h_" + str(int(height) * 2))

    return template.render(
        url=url,
        alt=alt,
        std_def_cloudinary_options=",".join(std_def_cloudinary_options),
        hi_def_cloudinary_options=",".join(hi_def_cloudinary_options),
        width=int(width),
        height=height,
        hi_def=hi_def,
        loading=loading,
        attrs=attrs,
    )


sys.modules[__name__] = image_template
